;*******************procedure circ_make version 1_0**************************
;Procedure	circ_make,c,h,f,image,ypix,peak,num,gapfiber
;
;Function   Count the number of line (traces) on the CCD image (image)
;           Identify the fiber number associated with each line
;           Identify the fiber numbers where the line is lost in
;           the gap between the two haves of the CCD
;
;Input:     median circfile header and image (c,h,f,image)
;Output:    ypix - array of y pixel values used for image cuts
;           peak - array of x-pixel value of line centres for image cuts
;           num  - Number of lines identified in circ image for each cut
;           gapfiber - Array identifying fibers in gap (max of 4)
;
;Report to log	Saving copy of Circ.gif
;Report files	Plot of image profiles showing the fiber number for each line (Circ.gif)
;
;Error   FATAL ERROR: unable to identify peaks correctly in image profile
;message FATAL ERROR: variable number of peaks found in image profile
;
;Procedures called;
;printl       - Prints text message to log file
;
;************************Procedure********************************************
pro circ_make,c,h,f,image,ypix,peak,num,gapfiber,fit_param

;**************set graphics window********************************************
; !P.MULTI =[0,1,c.circlev,0,0]
; scrollwindow,xsize=c.psize,ysize=fix(100.0*c.circlev)    &    !P.CHARSIZE=1.5

;******set up plot and output arrays******************************************
siz    = size(image)
peak   = fltarr(c.nfiber,c.circlev)
num    = intarr(c.circlev)
n_peak = 0L
p_array=fltarr(3,c.circlev)
yplot  = fltarr(c.circlev,siz(1))

;******evalue the size of frame in y **********************************
siz_y=fix(c.ymax-c.yzero)

;******set y-pixel value for intermediate cuts*********************************
ypix   = indgen(c.circlev)*float(siz_y)/float(c.ybin)/float((c.circlev-1))
;******set y-pixel value for upper and lower cuts******************************
ypix(0)= float(c.yavg)/float(c.ybin)

  	 if (c.circlev lt 6)   then printl,c.log,$
'FATAL ERROR Code 0: unable to start lines identification: Please increase the "circlev" value'
         if (c.circlev lt 6)   then stop
   	indup = fltarr(c.circlev,c.nfiber)
   	inddn = fltarr(c.circlev,c.nfiber)

;******create and analyse a smoothed profile at each cut level*****************
for k = 0,c.circlev-1 do begin

   	k_up = 0   &   k_dn = 0
	;************Select a strip accross the CCD image several y-pixels wide********
   
	y=image(*,ypix(c.circlev-1-k)-c.yavg/c.ybin:ypix(c.circlev-1-k)-1)

	;*********average and smooth this strip to give a line profile (y)*************
	y = smooth(median(y,dimension=2),c.circsmth/c.xbin)
   	scale  = mean(y)
	;*********Remove the low frequency component of the line profile***************
	
	
	if (strcmp(strtrim(h.ccdname,2),'REDPLUS4',/fold_case)) then begin
	  x=indgen(n_elements(y))
	  ynew=y
	  binsize=40.0/c.xbin
	  for binstep=min(x),max(x),binsize do begin
	    min_finder=where ((x gt binstep) and (x le binstep+binsize)  )
	    ynew(min_finder)=min(y(min_finder))
	  endfor
	  y2=y-smooth(ynew,(40.0/c.xbin))
	  y2[min(x):min(x)+(120.0/c.xbin)]=0.0
	  y2[max(x)-(120.0/c.xbin):max(x)]=0.0
	  y=y2
	  y=y-stddev(y)
	
	endif else begin
	
	  x=indgen(n_elements(y))
	  x_gap_ex=[x[0/c.xbin:2096/c.xbin],x[2216/c.xbin:4200/c.xbin]]
	  y_gap_ex=[y[0/c.xbin:2096/c.xbin],y[2216/c.xbin:4200/c.xbin]]
	  y[2096/c.xbin:2216/c.xbin]=spline(x_gap_ex,y_gap_ex,x[2096/c.xbin:2216/c.xbin])
	  ynew=y

	  binsize=40.0/c.xbin
	  for binstep=min(x),max(x),binsize do begin
	    min_finder=where ((x gt binstep) and (x le binstep+binsize)  )
	    ynew(min_finder)=min(y(min_finder))
	  endfor
	  y2=y-smooth(ynew,(40.0/c.xbin))
	  y2[min(x):min(x)+(120.0/c.xbin)]=0.0
	  y2[max(x)-(120.0/c.xbin):max(x)]=0.0
	  y2[2100/c.xbin:2212/c.xbin]=-10.0
	  y=y2
	  y=y-stddev(y)
	
	endelse
	
	;*********Normalise profile to its mean value**********************************
   	yplot(k,*) = y/scale
	;***********************plot current image profile*****************************
;    	plot,yplot(k,*),background=c.white,color=c.black,ytickinterval=2,yminor=1,xstyle=1,$
;      	yticklen =0.003,ytitle = 'y='+strtrim(ypix(c.circlev-1-k)-ypix(0)/2,2)
;    	oplot,y*0.0,color=c.black
	


        ;**********Fill array holding positive zero crossing values ******************
        ;;;;;;;;n_element(y) is on x direction (number of x pixels)   		
        for j = 1,n_elements(y)-1 do begin
      			if ((y(j-1) lt 0) and (y(j) ge 0)) and (k_up lt c.nfiber) then begin
         		indup(k,k_up) = j- (y(j))/float(y(j)-y(j-1))
         		k_up = k_up + 1   &     j=j+10/c.xbin
      			endif
   	endfor
	;**********Fill array holding negative zero crossing values ******************
 	for j = 1,n_elements(y)-1 do begin
 	
   			if ((y(j-1) ge 0) and (y(j) lt 0)) and (k_dn lt c.nfiber) then begin
         		inddn(k,k_dn) = j - (y(j))/float(y(j)-y(j-1))
        		k_dn = k_dn + 1     &      j=j+10/c.xbin
      			endif
	endfor

;;;;;;;;;;;;;;;;;;;create an array which hold information about each level;;;;;;;;;;;;;;;;;;;;;;;;;;;
p_array(*,c.circlev-k-1)=[k,k_up,k_dn]
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
endfor


;;;;;;;;;;;;;;;;;;;;;;find the more frequent fiber detected count and check it;;;;;;;;;;;;;;;;;
max_fiber=0
max_detect=transpose(p_array(1,*))
max_detect=max_detect(reverse(sort(max_detect)))
detect_index=max_detect*0
for i=0, n_elements(max_detect)-1 do begin 
fake_variable=where(max_detect eq max_detect(i), count)
detect_index(i)=count
endfor
if max(detect_index, index_max) gt 2 then max_fiber=max_detect(index_max(0)) else max_fiber=-1

if max_fiber le 0 then printl,c.log,$
'FATAL ERROR: unable to identify peaks correctly in image profile: Suggestion: Modify "circlev" and "yavg" parameters'
if max_fiber le 0 then stop
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;check cuts to find proper cuts to extract data;;;;;;;;;;;;;;;;;;;;;;;

eq_finder=where((p_array(1,*) eq p_array(2,*)) and (p_array(1,*) eq max_fiber) ,eq_count)
ne_finder=where((p_array(1,*) ne p_array(2,*)) or  (p_array(1,*) ne max_fiber) ,ne_count)

iter1_suc=(float(n_elements(eq_finder))/float(c.circlev))*100.0 

if (eq_count lt 4) then printl,c.log,$
         'FATAL ERROR: unable to identify peaks correctly in image profile: Suggestion: Modify "circlev" and "yavg" parameters'
if (eq_count lt 4) then stop


print, '**************************'
printl,c.log,'Iteration1: Lines(Fibers) Detection. (High S/N parts: Direct Detection)' 
print, ''
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;for cuts with correct values;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

if (eq_count ge 4) then begin
      for qq=0, n_elements(eq_finder)-1 do begin
         ;**Use +ve and -ve crossing values to fil array of x-pixel positions of peaks*
  	 num(c.circlev-p_array(0,eq_finder(qq))-1) = p_array(1,eq_finder(qq))
         for i = 0,c.nfiber-1 do peak(i,c.circlev-p_array(0,eq_finder(qq))-1) =$
         0.5*indup(p_array(0,eq_finder(qq)),i)+0.5*inddn(p_array(0,eq_finder(qq)),i)
         ; ;***********print number of lines identified in current cut********
        ; print,strtrim(fix(p_array(1,eq_finder(qq))),2)+'/'+strtrim(fix(p_array(2,eq_finder(qq))),2)+$
        ; ' lines identified at y = '+ strtrim(ypix(c.circlev-1-p_array(0,eq_finder(qq)))-float(c.yavg)/float(c.ybin)/2.,2)
;        ;  n_peak=num(0)      ;maximum number of peaks      
      endfor
n_peak=max(num(*))      ;maximum number of peaks 
endif
print, ''
printl,c.log,'Done!'
printl,c.log,strtrim(fix( n_peak ),2)+' lines Detected in high S/N parts.'
print, ''
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
print, '**************************'
printl,c.log,'Iteration2: Lines Detection. (Low S/N parts: Extrapolation)' 
print, 'Please wait... It may take up to a few minutes.' 

print, ''
;;;;;;for cuts with incorrect values (Use extrapolation to evaluate the peaks coordinates);;;;;;;
if (ne_finder(0) ne -1 ) then begin

        for nn=0, n_elements(ne_finder)-1 do begin
        
            x_fiber=ypix(eq_finder(*))-float(c.yavg)/float(c.ybin)/2.                       
            
            ;y_fiber=transpose(peak(1,(c.circlev-p_array(0,ne_finder(nn))):(c.circlev-1)))

            num(c.circlev-p_array(0,ne_finder(nn))-1)=n_peak(0)


	    
for fib=0, c.nfiber-1 do begin
;;;;extrapolation using cspline method
;peak(fib,c.circlev-p_array(0,ne_finder(nn))-1)=$cspline(x_fiber, transpose(peak(fib,(c.circlev-1-p_array(0,eq_finder(*))))), ypix(c.circlev-1-p_array(0,ne_finder(nn)))-float(c.yavg)/float(c.ybin)/2.)
;;;;extrapolation useing chebichev method


x_expect= ypix(c.circlev-1-p_array(0,ne_finder(nn)))-float(c.yavg)/float(c.ybin)/2.
y_fiber= transpose(peak(fib,(c.circlev-1-p_array(0,eq_finder(*)))))	    
peak(fib,c.circlev-p_array(0,ne_finder(nn))-1)=$
chebeval(x_expect, chebfit( x_fiber,y_fiber,nmax=fit_param,precision=1e-10,/quiet,REDUCE_ALGORITHM=0)) 

endfor
         
;print, strtrim(n_peak,2)+' lines identified at y=' +$
;strtrim(ypix(c.circlev-1-p_array(0,ne_finder(nn)))-float(c.yavg)/float(c.ybin)/2.,2)+' (Extrapolation)'
endfor          
endif


;;;;;;;;;;;;;;;;print the success rate in Initial Extrapolation ;;;;;;;;;

print, ''
printl,c.log,'Done!'
printl,c.log,strtrim(fix( n_peak ),2)+' lines Detected in Low S/N parts.'
iter1_suc_str=string(strtrim(iter1_suc,2), format='(F8.1)')
printl,c.log, 'Success Rate: '+iter1_suc_str+' %'
print, ''

; ;************check same number of lines found for each cut*********************
nline = n_peak(0)
index = where(num ne nline,count)
if count gt 0 then printl,c.log,$
'FATAL ERROR Code 3: variable number of peaks found in image profile: Modify "circlev" and "yavg" parameters'
if count gt 0 then stop
;***************count no. lines on each side of gap***************************
index=where(peak(0:nline-1,c.circlev/2) lt c.oscan2/c.xbin-1,nlhs)
index=where(peak(0:nline-1,c.circlev/2) gt c.oscan3/c.xbin-1,nrhs)
print,'with '+strtrim(nlhs,2)+' peaks on LHS  and '+strtrim(nrhs,2)+' on RHS'
;************************set array of all valid fibers ***********************
;******f.stat type 1 is a valid fiber & type 4 is a valid fiber but in gap****
;******f.type X is a fiber with invalid or parked target (i.e. no trace)******
;print, f.stat
fvalid = fix(f.stat)
fvalid(where (f.stat eq '1' or f.stat eq '4')) = 4
fvalid(where (f.type eq 'X')) = 0
;************identify fibers showing lines on rhs (i.e. not in gap)************
j = 1
for i = 1,c.nfiber do if f(c.nfiber-i).stat eq '1' and j le nlhs $
  and f(c.nfiber-i).type ne 'X'then begin
   fvalid(c.nfiber-i) = 1
   j = j + 1
endif
;*************identify fibers showing lines on lhs (i.e. not in gap**************
j = 1
for i = 0,c.nfiber-1 do if f(i).stat eq '1' and j le nrhs $
  and f(i).type ne 'X' then begin
   j = j + 1
   fvalid(i) = 1
endif
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;************capture fiber numbers of fibers in gap (maximum 4)******************
gapfiber = intarr(4)
j = 0
for i = 0,c.nfiber-1 do if fvalid(i) eq 4 and f(i).type ne 'X' then begin
  gapfiber(j) = i+1
  f(gapfiber(j)-1).stat='4'
  j = j + 1
endif

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;*************identify fiber numbers of lines (traces) on CCD image**************
f(*).line = 0
j = nline
for i = 0,c.nfiber-1 do if fvalid(i) eq 1 and f(i).type ne 'X' then begin
   f(i).line = j
   j = j - 1
endif
;*create array of fiber numbers corresponding to lines left to right across image*
fiber = intarr(nline)
j = 0
for i = 1,c.nfiber do if fvalid(c.nfiber-i) eq 1 then begin
   fiber(j) = c.nfiber-i+1
   j= j + 1
endif
;*************replot image profile with text indicating fiber number of each line*
for k = 0,c.circlev-1 do begin
;    plot,yplot(k,*),background=c.white,color=c.black,ytickinterval=2,yminor=1,xstyle=1,$
;      ytitle = 'y='+strtrim(ypix(c.circlev-1-k)-ypix(0)/2,2),yrange=[-2,8]
;    oplot,yplot(k,*)*0.0,color=c.black
;**************add fiber numbers to plot of mid y profile*************************
   if k eq (c.circlev/2) then for i = 0,num(0)-1 do begin
;       xx = peak(i,c.circlev-k-1)+siz(1)/500.
;       yy = (i-fix(i/2)*2)*5-0.5
;       XYOUTS,xx,yy,fiber(i),charsize=0.7,orientation=90,color=c.black
   endfor
endfor


print, '**************************'
printl,c.log,'Iteration3: Gaussian Peak Fitting.'
print, 'Please wait... It may take up to a few minutes.' 
print, ''

;**************** define the new ypix and peak*******************
;;;;;;;;;;experimental;;;;;;;;;;;;;;;;
ypix=ypix-float(c.yavg)/float(c.ybin)/2.0
peak=smooth(peak(*,*),[1,5])
peak2=(peak*0.0)-99.0
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
!p.multi = 0
;scrollwindow,xsize=600,ysize=700

; 	y=image(*,ypix(c.circlev-1-k)-c.yavg/c.ybin:ypix(c.circlev-1-k)-1)

;********define step size in x************
box_x=fix(10.0/c.xbin)
box_y=(float(c.yavg)/float(c.ybin)/2.)-1.0
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
iter_counter=0.
suc_counter=0.

for i = 0,nline-1 do begin 

  for j=0, c.circlev-1 do begin
    iter_counter=iter_counter+1.
    x_cent=fix(peak(i,j))
    y_cent=ypix(j)
    
    
    y=image[(x_cent-box_x):(x_cent+box_x),(y_cent-box_y):(y_cent+box_y)]
    x=findgen((box_x*2.0)+1.0)-(box_x)    
    y = smooth(median(y,dimension=2),c.circsmth/c.xbin)
    
    
;;; add an average in case of one pixel width in up 
    ;;;;;;;;this never happen;;;;;;;;;;;;;;
    if n_elements(x) ne n_elements(y) then printl,c.log,$
    'FATAL ERROR: Error in size of box in X and Y direction (Iter. 2)'
    if n_elements(x) ne n_elements(y) then stop
    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


      yfit = MPFITPEAK(X, Y, A, CHISQ=CHISQ, DOF=DOF, yerror=yerror, perror=perror)
      quality=yerror/stddev(y)


      if (a(0) lt 3.0*stddev(y) and abs(a(1)) lt (6.0/c.xbin) and quality lt 0.25 and perror[0]/a(0) lt 1.0 $
      and perror[1]/a(1) lt 1.0 and perror[2]/a(2) lt 1.0) $ 
      then  begin 
	peak2(i,j)=fix(peak(i,j))+a(1)
	suc_counter=suc_counter+1.
      endif
  endfor
endfor
iter2_suc=(suc_counter/iter_counter)*100.0

print, ''
printl,c.log,'Done!'

;;;;;;;;;;;;;;;;print the success rate in Gaussian fitting Iter.;;;;;;;;;
iter2_suc_str=string(strtrim(iter2_suc,2), format='(F8.1)')
printl,c.log, 'Success Rate: '+iter2_suc_str+' %'
print, ''
;;;;*****display the image and identify the peaks******** 
atv, image ,/histeq
for i = 0,nline-1 do begin 
atvplot, peak2(i,*),ypix(*), psym=2
      xx = peak(i,fix([0,(c.circlev/2.0),c.circlev]))-5.0
      yy = ypix(fix([0,(c.circlev/2.0),c.circlev]))
      atvXYOUTS,xx,yy,fiber(i),charsize=1.5,orientation=90
endfor

;;;****************Iteration 4********************

print, '**************************'
printl,c.log,'Iteration 4: Extrapolation Based on Third Iteration results'
print, 'Please wait... It may take up to a few minutes.' 
print, ''
for i = 0,nline-1 do begin 
  neg_finder=where(peak2(i,*) lt 0.0)
  pos_finder=where(peak2(i,*) gt 0.0)

  if (neg_finder(0) ne -1 and pos_finder(0) ne -1) then begin
    x_guas=ypix(pos_finder)
    x_exp=ypix(neg_finder)
    y_gaus=transpose(peak2(i,pos_finder))

    peak(i, neg_finder)=$
    chebeval(x_exp, chebfit( x_guas,y_gaus,nmax=fit_param,precision=1e-10,/quiet,REDUCE_ALGORITHM=0)) 
    peak(i, pos_finder)=peak2(i,pos_finder)
  endif else begin
    peak(i,*)=peak2(i,*)
  endelse

endfor
print, ''
printl,c.log,'Done!'
print, ''
;;;******************************************************

;*******************************Save plot to log file******************************
; WRITE_gif,c.pdir+c.plog+'Circ.gif',TVRD()
; printl,c.log,'Saving copy of Circ.gif'
; !p.multi = 0

for i = 0,nline-1 do atvplot, peak(i,*),ypix(*), linestyle=1, tick=1
circ_result=[transpose(ypix(*)),peak(*,*)]

;;;;;;;;;;;;;;;write the result in an ascii file;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
fname = strtrim(c.pdir+c.plog+'circ_result.txt',2)
OPENW,1,fname 
form=strcompress('('+strtrim(c.circlev,2)+'F9.3)',/remove_all)
PRINTF,1,transpose(circ_result(*,*)),FORMAT=form 
CLOSE,1

end
