;******************Procedure NSTAR version 1_0************************
;Procedure	NSTAR
;Function	Check the input file type, binning and fiber allocation are correct
;        	Create a median star image
;
;Input	   Raw data star files - series of raw data files (r1234567.fit)
;Output 	   nstar file (no1234567.fit) containing median star image
;
;Reports to log	RUNNING PROCEDURE:NSTAR
; 	               New NSTAR file created:ns1234567 median of # file
; 	               Saving copy of NSTAR.gif
;Report files	   Copy of median star image (star.gif)

;Error 	FATAL ERROR: incorrect file type, file type is xxx
;message WARNING: high noise in star files, ### adu
;     	ERROR: Saturation of star image (>99% full scale)
;     	WARNING: High star image, ## of full scale
;      	WARNING: Low star image, ## of full scale
;
;Procedures called;
;read_control - Returns control data in structure c
;read_data    - Reads pipeline header (ccns,hhns,ffns) and binary image of raw data file
;file_debias  - Reads pipeline header (ccns,hhns,ffns) and data array
;write_file   - Writes pipeline header (ccns,hhns,ffns) and data array to fits file
;printl       - Prints text message to log file
;
;*****************Procedure**************************************
pro NSTAR,noplot=noplot
;***************Read control data in structure c*****************
read_control,c
printl,c.log,'RUNNING PROCEDURE:NSTAR'

;******evalue the size of frame in y **********************************
siz_y=fix(c.ymax-c.yzero)



;**************clear open windows********************************
while (!d.window ge 0) do wdelete,!d.window
;***read header and debiased image of first star file in list****
;*******cancel operation if zero star files are specified***********
if c.starfile eq 0 then begin
printl,c.log,'FATAL ERROR: No star file found'
printl,c.log,'This Procedure needs at least one star file'
endif
if c.starfile eq 0 then stop
read_data,c.starfile,ccns,hhns,ffns,image
;***************Check file type**********************************
if (not strcmp(strtrim(hhns.obstype,2),'SKY')) and $
   (not strcmp(strtrim(hhns.obstype,2),'FLAT')) and $
  (not strcmp(strtrim(hhns.obstype,2),'TARGET'))then begin
   printl,c.log,'FATAL ERROR: incorrect file type, file type is '+h.obstype
   stop
endif
;***************creat median star file***************************
;*****define arrays to hold stack of star images and output******
siz = size(image)
inarr = fltarr(siz(1),siz(2),c.nstar)
outarr= fltarr(siz(1),siz(2))
scale = dblarr(c.nstar)

;*************load star images into stack array******************
for i = 0,c.nstar-1 do begin
;*********read header and debiased star image********************
   file_debias,c.dstar(i),ccns,hhns,ffns,image
;*******scale image to its mean value before taking median ******
   scale(i) = 1.0/mean(image)
   inarr(*,*,i) = image*scale(i)
endfor
;****define outarr as median of stack array and rescale**********
if c.nstar gt 1 then outarr = median(inarr,DIMENSION=3)/mean(scale) $
                                     else outarr = image
;*******save median image to star file (no1234567.fit)*********
write_file,c.pdir+c.pint+'ns'+strtrim(c.starfile,2),ccns,hhns,ffns,outarr
printl,c.log,'New star file created:ns'+strtrim(c.starfile,2)$
     +' median of '+strtrim(c.nstar,2)+' files'
;******* atv result ****************;
if not keyword_set(noplot) then atv, outarr
end
