pro pca_offset, osetfile

read_control,c
printl,c.log,'RUNNING PROCEDURE:pca_offset'   
;**********Return if no offset sky called or necessary files missing***
if c.skyoset ne 1 then printl,c.log,'Sky offset not called for'
if c.skyoset ne 1 then return
if c.noset eq 0   then printl,c.log,'FATAL ERROR: no offset sky files specified'
if c.noset eq 0  then stop   

;***********check offset sky file exists********************************
   openr,nr,osetfile+'.fit',/get_lun,ERROR=error
   if error ne 0 then begin
       printl,c.log,'FATAL ERROR: offset sky file o'+osetfile+' not found'
       stop
   endif
   close,nr    & free_lun,nr
   
;**********read the offset (sky) file ***********************************
read_file,osetfile,cc,h,f,mag,page=0
read_file,osetfile,cc,hh,ff,err,page=1

;**********find the number of offset fibers******************************
index = where(f.type eq 'P',count)
if count eq 0 then printl,$
clog,'FATAL ERROR: no target fibers found for sky offset'
if count eq 0 then stop
   
;*********create a matrix of offset spectra and related errors************ 
data=fltarr(c.nwl, count)
data_er=fltarr(c.nwl, count)


;;*********************create matrix of all sky fibers*********************
j=0
for i=1, c.nfiber do begin
if f(i-1).type eq 'P'then begin
data(*,j)=transpose(mag(i,*))
data_er(*,j)=transpose(err(i,*))
j=j+1
endif
endfor
;;*************************************************************************


printl,c.log,'PCA SKY MODE: SVD PCA (offset sky)'
;************* RUN PCA ALGORITHM ON OFFSET DATA****************************
;;-- number of PC amplitudes to calculate
namplitudes = 9
variances = 0 & evalues = 0 & pcs = 0 & espec = 0 & meanarr = 0
;params = {niter:20, nevec:18,nobj_init:10}
espec = vwpca(data,namplitudes,variances,pcs,meanarr,evalues,/svd,params=params)
; print, 'Top 10 variances (approx): ',variances[0:9],form='(A18,10(F0.4,1X))'
; print, 'Top 10 evalues  : ',evalues[0:9],form='(A18, 10(F0.4,1X))'

;*************Create new matrix holding result of PCA method ******************
mag_pca=fltarr(c.nwl, c.nfiber+2)
j=0
for i=1, c.nfiber do begin
if f(i-1).type eq 'P' then begin
mag_pca(*,i)=(vwpca_reconstruct(pcs[*,j],espec[*,0:namplitudes-1])+meanarr) ;+data_c(*,j)

j=j+1
endif else begin
mag_pca(*,i)=0.
endelse
endfor

mag_pca=transpose(mag_pca)
mag_pca(0,*)=mag(0,*)

;;;;;it must analyzed later;;;;;;;;;;;;;
err_pca=err
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


;Save output spectra to fits file with page 1 for mag and page 2 for err
write_file,c.pdir+c.pspec+'pca_o'+strtrim(c.osetfile,2),c,h,f,mag_pca,page=0
write_file,c.pdir+c.pspec+'pca_o'+strtrim(c.osetfile,2),c,h,f,err_pca,page=1
write_file,c.pdir+c.pspec+'iraf_pca_o'+strtrim(c.osetfile,2),c,h,f,mag_pca,page=0, /export
write_file,c.pdir+c.pspec+'iraf_pca_o'+strtrim(c.osetfile,2),c,h,f,err_pca,page=1, /export
;*****Messages to screen and logfile showing module actions done*******
printl,c.log, $
'Median offset (PCA algorithm) sky spectra saved to file pca_o'+strtrim(c.doset(0),2)

end