;***********Procedure read_control version 1_0*******************
;Procedure:read_control,c
;Function:Read control data set (c) from ASCII file named*********
;         control_data.txt located in the pipeline directory
;Option:  read_control,c,file=xxx read control data from a file xxx
;         where xxx is the  file name including path
;
;Input   :None
;Output  :control data in structure (c)
;
;Procedures called;readcol_af2.pro
;
;*****local function used to set consistent path separator(/)*****
function separater,path
for i = 0,10  do if  path_sep() ne '/' and strpos(path,'/') ge 0 then $
   strput,path,'\',strpos(path,'/')
return, path
end
;
;*****************procedure****************************************
pro read_control,c,FILE=file
;*******set structure for control data structure*******************
;**descriptors of control data are insection 1.1 of developer notes
c = {c,rbname:'x',pdir:'x',fiberid:0,dfiber1:0,dfiber2:0,circcutl:0$
      ,circcutr:0,skyoset:0,skytype:0,wlmin:0.0D,wlmax:0.0D$
      ,fibrattn:0,profile:0,flattype:0,nscatter:0,wltype:0$
      ,xbin:0,ybin:0,flatsmth:20,ypixel:0,yzero:0,ymax:0$
      ,maskcut:0.0,maskmax:0.0,masklev:0.0$
      ,biaserr:0.0,flaterr:0.0,sighi:0.0,siglo:0.0$
      ,extsigma:0,skymin:0.0,skymax:0.0,msigmax:0.0$
      
      ,ccdron:0.0D,ccdgain:0.0D,ccdzero:0L$
      ,noscan:0,oscan1:0,oscan2:0,oscan3:0,oscan4:0$
      ,pixelsiz:0.0,csize:0.0,circlev:0,circsmth:0$
      ,circzero:0.0,lwide:0,yavg:0,boxwide:0,staritn:0,nwl:0L$
      
      ,nfiber:0,nfiducl:0,rcircle:0.0D,dfiber3:0,dfiber4:0$
      ,fiberdip:0.0,fiberoff:0.0$
      
      ,pdata:'x',pint:'x',pspec:'x',plog:'x',ptemp:'x',pipdir:'x'$

      ,arcorder:0,arcwide:0,arcrefc:0,arcrefw:0,arcscale:0,afilecut:0$
      ,atlfile:'x',natl:0,atlscale:0,atlflag:0,atlaline:0$

      ,log:'x',nparam:0,flag1:0,flag2:0,flag3:0,flag4:0$
      ,atl:fltarr(4),atlwl:dblarr(50)$
      ,psize:0,black:0L,red:0L,blue:0L,white:0L$

      ,biasfile:0L,nbias:0,dbias:lonarr(99)$
      ,flatfile:0L,nflat:0,dflat:lonarr(99)$
      ,maskfile:0L,nmask:0,dmask:lonarr(99)$
      ,circfile:0L,ncirc:0,dcirc:lonarr(99)$
      ,arcfile :0L,narc :0,darc :lonarr(99)$
      ,starfile:0L,nstar:0,dstar:lonarr(99)$
      ,osetfile:0L,noset:0,doset:lonarr(99)$
      ,narcline:0,acalwl:dblarr(15)$
      ,apixc:lonarr(15),apixw:lonarr(15),aflag:intarr(15)}

;**************Open control_data file*****************************
if keyword_set(file) then begin
readcol_af2, file ,v1,v2, delimiter=';',$
format='A,A', STRINGSKIP='#', /silent 
endif else begin
readcol_af2, 'control_data.txt',v1,v2,$
delimiter=';', format='A,A' , STRINGSKIP='#', /silent
endelse

;******read single parameter items of control data****************
;*****note parameters are read in the order they are found in*****
;*therefore the contol file mist contain a list of all parameters
;*in the correct order with no blank lines************************
;****any comments following a semi-colon are not read*************

dummy = 'txt'
num   = 0
val   = 0.0D
lnum=0L
fval=0.0

;;;;;;;;;;;;;check path separator base on os version;;;;;;;;;;;;
if strcmp(strupcase(!version.os_family),'UNIX') then sep_char='/'
if strcmp(strupcase(!version.os_family),'WINDOWS') then sep_char='\'
;;;;;;;;;;read working directroies paths (string);;;;;;;;;;;
str1_array=['PDIR','PDATA','PINT','PSPEC','PLOG','PTEMP']

for i = 0,(n_elements(str1_array)) - 1 do begin
dummy_var=where(strcmp(STRTRIM(v2,2),str1_array(i), /FOLD_CASE) eq 1)
if (dummy_var eq -1) or (n_elements(dummy_var) gt 1) then print,$
strtrim('FATAL ERROR: '+str1_array(i)+$
' keyword not found or duplicate entries found in the Configuration file',2)
if (dummy_var eq -1) or (n_elements(dummy_var) gt 1) then stop

status = TAG_EXIST(c, STRTRIM(str1_array(i),2), index=index)
if status eq 1 then c.(index)=STRTRIM(v1(dummy_var),2)
if status ne 1 then print, strtrim('Warning: '+str1_array(i)+$
' is set to its default value',2)
if strcmp(strmid(c.(index),0,1,/reverse_offset),sep_char) ne 1 then print,$
strtrim('Fatal Error: Check '+str1_array(i)+$
' directory path format in your configuration file.'+$
'You may forgot to add / (unix) or \ (windows) to the end of path',2)
if strcmp(strmid(c.(index),0,1,/reverse_offset),sep_char) ne 1 then stop
endfor

;;;;;;;;;;;;experimental part;;;;;;;;;;;;;;;;;;;
;;;;;;;;folders and paths validation;;;;;;;;;;;;
pdir_valid = FILE_INFO(c.pdir)
if pdir_valid.exists ne 1 then print, $
strtrim('Fatal Error: '+c.pdir+' is not a valid path.',2)
if pdir_valid.exists ne 1 then stop

data_folder_check = $
FILE_SEARCH(strtrim(c.pdir+c.pdata+'*',2),count=count_in_pdata)
if count_in_pdata lt 1 then print,$
strtrim('Fatal Error: There is no data within '+c.pdir+c.pdata+' directory',2)
if count_in_pdata lt 1 then stop

;;;;create subdirectories ;;;;;;;;;;;;;;;;;;;;;;
dir_array=[c.pint,c.pspec,c.plog,c.ptemp]
dir_array_name=STRMID(dir_array,0,transpose(strlen(dir_array)-1))
FILE_MKDIR, c.pdir+dir_array

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;



;;;;;;;;;;read project name and Calibration Lamp file name (string);;;;;;;;;;;
str2_array=['RBNAME','ATLFILE']

for i = 0,(n_elements(str2_array)) - 1 do begin
dummy_var=where(strcmp(STRTRIM(v2,2),str2_array(i), /FOLD_CASE) eq 1)
if (dummy_var eq -1) or (n_elements(dummy_var) gt 1) then print,$
strtrim('FATAL ERROR: '+str2_array(i)+$
' keyword not found or duplicate entries found in the Configuration file',2)
if (dummy_var eq -1) or (n_elements(dummy_var) gt 1) then stop

status = TAG_EXIST(c, STRTRIM(str2_array(i),2), index=index)
if status eq 1 then c.(index)=STRTRIM(v1(dummy_var),2)
if status ne 1 then print, strtrim('Warning: '+$
str2_array(i)+' is set to its default value',2)
endfor

;;;;;;;;;;;;;;;;;;;;;;;;;;read int data;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

int_array=['FIBERID','DFIBER1','DFIBER2','SKYOSET','SKYTYPE','FIBERID',$
'FIBRATTN','PROFILE','NSCATTER','WLTYPE','XBIN','YBIN',$
'CIRCSMTH','LWIDE','YAVG','BOXWIDE','STARITN','FLATSMTH',$
'EXTSIGMA','YPIXEL','YZERO','YMAX','NOSCAN','OSCAN1','OSCAN2',$
'OSCAN3','OSCAN4','CIRCCUTL','CIRCCUTR','NFIBER','NFIDUCL','DFIBER3',$
'DFIBER4','ARCORDER','ARCWIDE','ARCREFC','ARCREFW','ARCSCALE','AFILECUT',$
'NATL','ATLSCALE','ATLFLAG','ATLALINE','NBIAS','NFLAT','NMASK','NCIRC',$
'NARC','NSTAR','NOSET','NARCLINE']

for i = 0, (n_elements(int_array)) - 1 do begin
dummy_var=where(strcmp(STRTRIM(v2,2),int_array(i), /FOLD_CASE) eq 1)
if (dummy_var eq -1) or (n_elements(dummy_var) gt 1) then print,$
strtrim('FATAL ERROR: '+int_array(i)+$
' keyword not found or duplicate entries found in the Configuration file',2)
if (dummy_var eq -1) or (n_elements(dummy_var) gt 1) then stop

status = TAG_EXIST(c, STRTRIM(int_array(i),2), index=index)
if status eq 1 then c.(index)=fix(STRTRIM(v1(dummy_var),2))
if status ne 1 then print, strtrim('Warning: '+$
int_array(i)+' is set to its default value',2)
endfor

;;;;;;;;;;;;;;;;;;;;;;;read long int data;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

long_array=['CCDZERO']
for i = 0,(n_elements(long_array)) - 1 do begin
dummy_var=where(strcmp(STRTRIM(v2,2),long_array(i), /FOLD_CASE) eq 1)
if (dummy_var eq -1) or (n_elements(dummy_var) gt 1) then print,$
strtrim('FATAL ERROR: '+long_array(i)+$
' keyword not found or duplicate entries found in the Configuration file',2)
if (dummy_var eq -1) or (n_elements(dummy_var) gt 1) then stop

status = TAG_EXIST(c, STRTRIM(long_array(i),2), index=index)
if status eq 1 then c.(index)=long(STRTRIM(v1(dummy_var),2))
if status ne 1 then print, strtrim('Warning: '+$
long_array(i)+' is set to its default value',2)
endfor
;;;;;;;;;;;;;;;;;;;;;;;;;read double data;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
double_array=['WLMIN','WLMAX','CCDRON','CCDGAIN','RCIRCLE']

for i = 0,(n_elements(double_array)) - 1 do begin
dummy_var=where(strcmp(STRTRIM(v2,2),double_array(i), /FOLD_CASE) eq 1)
if (dummy_var eq -1) or (n_elements(dummy_var) gt 1) then print,$
 strtrim('FATAL ERROR: '+double_array(i)+$
 ' keyword not found or duplicate entries found in the Configuration file',2)
if (dummy_var eq -1) or (n_elements(dummy_var) gt 1) then stop

status = TAG_EXIST(c, STRTRIM(double_array(i),2), index=index)
if status eq 1 then c.(index)=double(STRTRIM(v1(dummy_var),2))
if status ne 1 then print, strtrim('Warning: '+$
double_array(i)+' is set to its default value',2)
endfor


;;;;;;;;;;;;;;;;;;;;;;;;read float data;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
float_array=['CSIZE','MASKCUT','MASKMAX','MASKLEV','CIRCZERO',$
'BIASERR','FLATERR','SIGHI','SIGLO','SKYMIN','SKYMAX','MSIGMAX',$
'FIBERDIP','FIBEROFF','PIXELSIZ']

for i = 0,(n_elements(float_array)) - 1 do begin
dummy_var=where(strcmp(STRTRIM(v2,2),float_array(i), /FOLD_CASE) eq 1)
if (dummy_var eq -1) or (n_elements(dummy_var) gt 1) then print,$
strtrim('FATAL ERROR: '+float_array(i)+$
' keyword not found or duplicate entries found in the Configuration file',2)
if (dummy_var eq -1) or (n_elements(dummy_var) gt 1) then stop

status = TAG_EXIST(c, STRTRIM(float_array(i),2), index=index)
if status eq 1 then c.(index)=float(STRTRIM(v1(dummy_var),2))
if status ne 1 then print, strtrim('Warning: '+$
float_array(i)+' is set to its default value',2)
endfor


;;;;;;;;;;;;;;;;;;;;;;;;;;;read data frames;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;  

list1_array=['DBIAS','DFLAT','DMASK','DCIRC','DARC','DSTAR',$
'DOSET','ACALWL','APIXC','APIXW','AFLAG']
list2_array=[c.nbias,c.nflat,c.nmask,c.ncirc,c.narc,c.nstar,$
c.noset,c.narcline,c.narcline,c.narcline,c.narcline]


for i = 0,(n_elements(list1_array)) - 1 do begin

co_var=where(strcmp(STRTRIM(v2,2),list1_array(i),$
strlen(list1_array(i)), /FOLD_CASE) eq 1)

if (co_var(0) eq -1) and (list2_array(i) gt 0) then print,$
 strtrim('FATAL ERROR: '+list1_array(i)+$
 ' keyword not found or duplicate entries found'+$
'Ambiguous entries found.'+$
'check number of frames and related parameters in the Configuration file',2)
if (co_var(0) eq -1) and (list2_array(i) gt 0) then stop                

status = TAG_EXIST(c, STRTRIM(list1_array(i),2), index=index)
if (status eq 1) and (n_elements(co_var) ge list2_array(i)) and $
(list2_array(i) gt 0) then begin
for j = 0,(list2_array(i)) - 1 do begin
if list1_array(i) eq 'ACALWL'  then $
c.(index)(j) = double(strtrim(v1(co_var(j)),2))  
if list1_array(i) eq 'AFLAG'   then $
c.(index)(j) = fix(strtrim(v1(co_var(j)),2))  
c.(index)(j) = long(strtrim(v1(co_var(j)),2))
endfor

endif
if (status ne 1) or (n_elements(co_var) lt list2_array(i)) then print,$
strtrim('Warning: '+list1_array(i)+': '+$
'Ambiguous entries found. '+$
'Check number of frames and related parameters in the Configuration file.',2)
endfor

;;;;;Assign the first frame of each set;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
c.biasfile = c.dbias(0)
c.flatfile = c.dflat(0)
c.maskfile = c.dmask(0)
if c.dcirc(0) eq 0 then c.circfile = c.dflat(0) else c.circfile = c.dcirc(0)
c.arcfile  = c.darc(0)
c.starfile = c.dstar(0)
c.osetfile = c.doset(0)







;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; 

;****Ensure that all file paths specify a consistent separator
 c.pdata = separater(c.pdata)
 c.pint  = separater(c.pint)
 c.plog  = separater(c.plog)
 c.pspec = separater(c.pspec)
 c.ptemp = separater(c.ptemp)
;*set total numbers for different types of control parameters*
;**these values are used to set table size in the GUI********
c.flag1  = 64        ;No  basic parameters
c.flag2  = 11        ;No arc parameters
;***set additional control parameters used in I/O procedures*
c.log = c.pdir+c.plog+c.rbname+'_log.txt'
c.nparam  = c.flag1 + c.flag2
;********set plot window parameters**************************
;**this is required to reset IDL settings after procedure ATV
;**has been run since can leave things in a strange state****
if strcmp(strupcase(!version.os_family),'UNIX') then set_plot,'X'
if strcmp(strupcase(!version.os_family),'WINDOWS') then set_plot,'WIN'
device,decomposed=1
;set parameter controlling scaling of plot window on screen*
;***c.psize sets the maximum pixel size used****************
;**according to the screen size (with a maximum of 1000)****
c.psize = 1000
if (get_screen_size())[0] lt c.psize then $
          c.psize=fix(0.9*(get_screen_size())[0])
;********set colours used for pipeline plot******************
siz = long(!d.table_size)
tvlct,bindgen(siz),bindgen(siz),bindgen(siz)
c.black = 1
c.red   = long(180.0*siz/256.0)
c.blue  = (siz-1)*siz + c.red*siz*siz
c.white = siz*siz*siz-1

;;;;;;;;;;;;;;;We are trying to remove some parameters;;;;;;;;
;;;;;;;;;;;;;;; from this configuration file;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;and fix them to their optimal values;;;;;;;;;;;
c.circlev=100
c.flattype=0

avg_y=(fix(c.ymax-c.yzero)/fix(c.ybin*(c.circlev+1)))*fix(c.ybin)
if avg_y gt 24 then c.yavg=24 else c.yavg=avg_y

if c.flatsmth lt 10 then c.flatsmth=10
if c.staritn  gt 1  then c.staritn =1


;;;;;;;;;;;;;;;;;;;;;;change the definition of c.nwl;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;in star routine it will be changed based on offset (in y) of fibers;;;;;
c.nwl=fix(c.ymax-c.yzero)/fix(c.ybin)
;c.nwl=10000
;;;;;;;;this routine looks for pipeline directory;;;;;;;;;;;;;
;;;;;;;;;;;;;;and returns its path as c.pipdir;;;;;;;;;;;;;;;;

this_file=strtrim('read_control.pro',2)
adr1 = strtrim(FILE_WHICH(this_file, /INCLUDE_CURRENT_DIR),2)
if strlen(adr1) ge strlen(this_file) then begin
c.pipdir=strmid(adr1, 0,(strlen(adr1)-strlen(this_file)))
endif else begin
c.pipdir=c.pdir
print,$
strtrim('FATAL ERROR: Pipline directory not found ',2)
stop
endelse

;;update the control file;;;
write_control,c
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
end
