;************Procedure sky_sub version 1_0******************************
;Procedure  sky_sub,nf,c,f,wl,starspec,starerr,skymed,skyerr,sscale
;
;Function   Subtract sky spectrum from star spectrum.
;            c.skytype=0:No sky spectrum is subtracted
;            c.skytype=1:linear subtraction of sky spectrum
;            c.skytype=2:sky spectrum scaled for 'optimum' subtraction, using
;            scale of sky emission lines.
;            c.skytype=3:sky spectrum scaled and output masked over large sky lines
;            c.skytype=-2:sky spectrum scaled for 'optimum' subtraction using old method
;            c.skytype=-3:sky spectrum scaled and output masked over large sky lines
;            based on old scaling method.
;
;Input:     fiber number counter (i) and header data (c,f)
;           common wavelegth scale (wl), star spectrum and its error
;           sky spectrum and its error
;Output:    sky subtracted star spectrum and error
;           header data updated to show fraction of sky spectrum subtractedtype of 
;           sky subtraction used set by parameter c.skytype and scale if necessary;
;
;Error    WARNING poor convergence of sky subtraction for fiber ###, scale parameter set to 1
;Message
;
;Procedures called:
;printl       - Prints text message to log file
;
;*****************Procedure**********************************************
pro sky_sub,nf,c,f,wl,starspec,starerr,skymed,skyerr,sscale
;***for skytype 0 set header f.sky=0  and return (with no subraction)****
f(nf).sky = 0
sscale=f(nf).sky
if c.skytype eq 0 then return
;********hold unmodified version of starspec in starspco*****************
starspeco = starspec
;*********linear subtraction of sky spectrum*****************************
starspec  = starspeco - skymed
;************add error in sky spectrum in quadrature*********************
starerr   = sqrt(starerr*starerr + skyerr*skyerr)
;*******for skytype 1 set header f.sky=1  and return**********************
f(nf).sky = 1
sscale=f(nf).sky
if c.skytype eq 1 then return


;;;******* new sky subtraction (finding scale based on sky peaks)********
if c.skytype ge 2 then begin
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
raw_wave=     wl
sky_wave=     wl
raw_spec=     starspeco
sky_spec=     skymed
raw_err_spec= starerr
sky_err_spec= skyerr
;;;*************** evaluating the Continuum ****************;;

raw_dc = im_fitcontinuum(raw_wave, raw_spec, raw_err_spec,method=2, /silent)
sky_dc = im_fitcontinuum(sky_wave, sky_spec, sky_err_spec,method=2, /silent)

;;;********** excluding the Continuum from spectra***********;;
;;;***************smoothing the ACs components **************;;
raw_ac = raw_spec-(raw_dc)
sky_ac = sky_spec-(sky_dc)
skyo=sky_ac
raw_ac=smooth(raw_ac , (10.0/c.ybin) , /EDGE_TRUNCATE)
sky_ac=smooth(sky_ac , (10.0/c.ybin) , /EDGE_TRUNCATE)


;*********linear subtraction of sky spectrum*****************************
starspec = raw_spec - sky_spec
starspec_err=sqrt(((raw_err_spec)^2.0)+((sky_err_spec)^2.0))
starspec_dc=im_fitcontinuum(raw_wave, starspec, starspec_err,method=2, /silent)


;**********take ac component of sky subtracted spectrum******************
starspec_ac = starspec - starspec_dc
starspec_ac=smooth(starspec_ac , (10.0/c.ybin) , /EDGE_TRUNCATE)
;*******get zero offset convolution of with ac sky spectrum**************
conv_unit = mean(starspec_ac*sky_ac)
;test1=starspec_ac*sky_ac
;;*********** Finding the peaks in sky spectra **************;;
;;*****Evaluate the scale for peaks and raw data ************;;
peak_sky=peaks(sky_ac,2)
;   print, peak_sky
if n_elements(peak_sky) gt 3.0 then begin
	peak_scl =  median(raw_ac(peak_sky)/sky_ac(peak_sky))
;	print, peak_scl
endif else begin
   peak_scl=1.0
   printl,c.log,'WARNING poor convergence of sky subtraction for fiber '+$
   strtrim(nf+1,2)+', scale parameter set to 1'
endelse

;**set f.sky to achieve to a zero convolution with ac sky spectrum*******
f(nf).sky = peak_scl
sscale=f(nf).sky
;*******subtract this optimum fraction of the sky spectrum***************
starspec = raw_spec - f(nf).sky*sky_spec
;***********************reject poor convergence**************************
;**********take ac component of sky subtracted spectrum******************
;starspec_dc=im_fitcontinuum(raw_wave, starspec, starspec_err,method=2)
;starspec_ac = starspec - starspec_dc
;starspec_ac=smooth(starspec_ac , (10.0/c.ybin) , /EDGE_TRUNCATE)

;*******get zero offset convolution of with ac sky spectrum**************
;conv_fin = mean(starspec_ac*sky_ac)
;*******check for convergence********************************************
;if abs(conv_fin) gt abs(conv_unit) then begin
;   printl,c.log,'WARNING poor convergence of sky subtraction for fiber '+$
;    strtrim(nf+1,2)+', scale parameter set to 1'
;;;***********In case of poor convergence set scaling parameter f.sky=1****
;   f(nf).sky = 1
;   sscale=f(nf).sky
;   starspec = raw_spec - f(nf).sky*sky_spec
;endif

endif

;;;;; If user want to use old sky subtraction (old option2) method;;;;;
if c.skytype le -2 then begin
;*********************optimum sky subtraction****************************
;**********take ac component of sky spectrum*****************************
skyo  = skymed   - robust_mean(skymed,5)
;**********take ac component of original star spectrum*******************
staro = starspeco - robust_mean(starspeco,5)
;*******get zero offset convolution of these two spectra*****************
conv_zero = mean(staro*skyo)
;*********linear subtraction of sky spectrum*****************************
starspec = starspeco - skymed
;**********take ac component of sky subtracted spectrum******************
star = starspec - robust_mean(starspec,5)
;*******get zero offset convolution of with ac sky spectrum**************
conv_unit = mean(star*skyo)
;**set f.sky to achieve to a zero convolution with ac sky spectrum*******
f(nf).sky = 1 + conv_unit/conv_zero
sscale=f(nf).sky
;*******subtract this optimum fraction of the sky spectrum***************
starspec = starspeco - f(nf).sky*skymed
;***********************reject poor convergence**************************
;**********take ac component of sky subtracted spectrum******************
star = starspec - robust_mean(starspec,5)
;*******get zero offset convolution of with ac sky spectrum**************
conv_fin = mean(star*skyo)
;*******check for convergence********************************************
if abs(conv_fin) gt abs(conv_unit) then begin
   printl,c.log,'WARNING poor convergence of sky subtraction for fiber '+$
    strtrim(nf+1,2)+', scale parameter set to 1'
;***********In case of poor convergence set scaling parameter f.sky=1****
   f(nf).sky = 1
   sscale=f(nf).sky
   starspec = starspeco - f(nf).sky*skymed
endif

endif

;*******for skytype 2 return*********************************************
if (c.skytype eq 2) or (c.skytype eq -2) then return

;*****Mask sky lines above level specified in control data***************
mask = intarr(n_elements(skymed))
mask(*) = 1
;************************Mask sky lines**********************************
;**define masking vector set to zero when sky signal exceed cut off level
index = where(skyo gt c.skymax,count)
if count gt 0 then mask(index) = 0
;**broaden masked sections to allow for any misalignment at line edges**
nwl = n_elements(wl)
mask(2:nwl-1) = mask(2:nwl-1)*mask(0:nwl-3)
mask(0:nwl-3) = mask(0:nwl-3)*mask(2:nwl-1)
;**********mask sky subtracted spectrum*********************************
starspecm = starspec*mask
starerrm  = starerr*mask
;******************************Interpolate masked data******************
;************set first and last point of mask to 1**********************
mask(0) = 1     &     mask(n_elements(mask)-1) = 1
;Set arrays to hold  pixel No., magnitude and variance of unmasked points
tempw = fltarr(n_elements(wl))
temps = fltarr(n_elements(wl))
tempe = fltarr(n_elements(wl))
;**initialise counter of number of pixels containing unmasked point*******
 j=0
;***fill arrays containing pixel No., mag and val for unmasked point******
for m = 0,n_elements(wl)-1 do begin
   tempw(j) = wl(m)
   temps(j) = starspecm(m)
   tempe(j) = starerrm(m)
 ;***increment counter if original spectra contains unmasked point**********
   if mask(m) ne 0 then j = j + 1
endfor
 ;*truncate arrays of wavelength., mag and var at max No. of unmasked points
tempw = tempw(0:j-1)
temps = temps(0:j-1)
tempe = tempe(0:j-1)
;*****interpolate data for unmasked points over all wavelength values*******
starspec=SPLINE(tempw,temps,wl)
starerr =SPLINE(tempw,tempe,wl)
end
