;*********Procedure write_control version 1_0*****************
;Procedure:write_control,c
;Function:Write control data set (c) to ASCII file named*********
;         control_data.txt located in the pipeline directory
;Option:  read_control,c,file=xxx write control data to  file xxx
;         where xxx is the  file name including path
;
;Input   :None
;Output  :control data in structure (c)
;
;Procedures called: match2
;
;************Procedure**********************************************
pro write_control,c,file=FILE
if keyword_set(file) then openw, nw,file, /get_lun else $
             openw, nw,'control_data.txt', /get_lun
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;**********define a vector of keyword which we want to access them indivisulay********
forbid=['RBNAME','PDIR','LOG','PDATA','PINT','PSPEC','PLOG','PTEMP',$
'NBIAS','DBIAS','NFLAT','DFLAT','NMASK','DMASK','NCIRC',$
'DCIRC','NARC','DARC','NSTAR','DSTAR','NOSET','DOSET','NARCLINE',$
'ACALWL','APIXC', 'APIXW', 'AFLAG','ATL', 'ATLWL','BLACK','RED',$
'BLUE','WHITE','FLAG1','FLAG2','FLAG3','FLAG4','NPARAM','PSIZE','PIPDIR', 'NWL',$
'FLATTYPE', 'CIRCLEV', 'BIASFILE', 'FLATFILE','MASKFILE','CIRCFILE','ARCFILE',$
'STARFILE','OSETFILE','ARCORDER','ARCWIDE','ARCREFC','ARCREFW','ARCSCALE',$
'AFILECUT', 'ATLFILE', 'NATL', 'ATLSCALE', 'ATLFLAG', 'ATLALINE']

tag = tag_names(c)
match2, tag, forbid, tag1,forbid1
non_match=where(tag1 eq -1)

;**********define formats of outputs********************************
f0='(A)'
f1='(A-22,A-4,A)'
length_pdir=strlen(string(c.(where(tag(*) eq 'PDIR'))))
f2='(A-'+strtrim((length_pdir+10),2)+',A-4,A)'

;**********start writing file *****************

printf, nw, '  ', format=f0

printf, nw, '#********|    AF2 pipeline: Configuration file  |********* #', format=f0


printf, nw, '  ', format=f0
printf, nw, '  ', format=f0
printf, nw, '#****************|   Project Directory   |*****************#', format=f0
printf, nw, '  ', format=f0

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
printf,nw,strtrim((c.(where(tag(*) eq 'RBNAME'))), 2 ),';' ,string(tag(where(tag(*) eq 'RBNAME'))), format=f2
printf,nw,strtrim((c.(where(tag(*) eq 'PDIR'))), 2 )  ,';' ,string(tag(where(tag(*) eq 'PDIR')))  , format=f2

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
printf, nw, '  ', format=f0
printf, nw, '#*************|   Project Sub-directories   |**************#', format=f0
printf, nw, '  ', format=f0
printf,nw,strtrim((c.(where(tag(*) eq 'PDATA'))), 2 )   ,';' ,string(tag(where(tag(*) eq 'PDATA')))   , format=f1
printf,nw,strtrim((c.(where(tag(*) eq 'PINT'))), 2 )    ,';' ,string(tag(where(tag(*) eq 'PINT')))    , format=f1
printf,nw,strtrim((c.(where(tag(*) eq 'PSPEC'))) , 2 )  ,';' ,string(tag(where(tag(*) eq 'PSPEC')))   , format=f1
printf,nw,strtrim((c.(where(tag(*) eq 'PLOG'))) , 2 )   ,';' ,string(tag(where(tag(*) eq 'PLOG')))    , format=f1
printf,nw,strtrim((c.(where(tag(*) eq 'PTEMP'))), 2 )   ,';' ,string(tag(where(tag(*) eq 'PTEMP')))   , format=f1


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
printf, nw, '  ', format=f0
printf, nw, '#********************|   Parameters   |********************#', format=f0
printf, nw, '  ', format=f0
for i = 0, n_elements(non_match)-1 do begin 

printf,nw,strtrim((c.(non_match(i))), 2 ),';' ,string(tag(non_match(i))), format=f1

endfor

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
printf, nw, '  ', format=f0
printf, nw, '#********************|   Data Types   |*********************#', format=f0
printf, nw, '  ', format=f0
printf,nw,strtrim((c.nbias), 2 ),';','NBIAS', format=f1
for i = 0,c.nbias-1 do printf,nw,strtrim((c.dbias(i)), 2 ),';','DBIAS', format=f1

printf,nw,strtrim((c.nflat), 2 ),';','NFLAT', format=f1
for i = 0,c.nflat-1 do printf,nw,strtrim((c.dflat(i)), 2 ),';','DFLAT', format=f1

printf,nw,strtrim((c.nmask), 2 ),';','NMASK', format=f1
for i = 0,c.nmask-1 do printf,nw,strtrim((c.dmask(i)), 2 ),';','DMASK', format=f1

printf,nw,strtrim((c.narc), 2 ),';','NARC', format=f1
for i = 0,c.narc-1 do printf,nw,strtrim((c.darc(i)), 2 ),';','DARC', format=f1

printf,nw,strtrim((c.ncirc), 2 ),';','NCIRC', format=f1
for i = 0,c.ncirc-1 do printf,nw,strtrim((c.dcirc(i)), 2 ),';','DCIRC', format=f1

printf,nw,strtrim((c.nstar), 2 ),';','NSTAR', format=f1
for i = 0,c.nstar-1 do printf,nw,strtrim((c.dstar(i)), 2 ),';','DSTAR', format=f1

printf,nw,strtrim((c.noset), 2 ),';','NOSET', format=f1
for i = 0,c.noset-1 do printf,nw,strtrim((c.doset(i)), 2 ),';','DOSET', format=f1

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
printf, nw, '  ', format=f0
printf, nw, '#***************|   Calibration Parameters   |***************#', format=f0
printf, nw, '  ', format=f0

printf,nw,strtrim((c.arcorder), 2 ),';','ARCORDER', format=f1
printf,nw,strtrim((c.arcwide), 2 ),';','ARCWIDE', format=f1
printf,nw,strtrim((c.arcrefc), 2 ),';','ARCREFC', format=f1
printf,nw,strtrim((c.arcrefw), 2 ),';','ARCREFW', format=f1
printf,nw,strtrim((c.arcscale), 2 ),';','ARCSCALE', format=f1
printf,nw,strtrim((c.afilecut), 2 ),';','AFILECUT', format=f1
printf,nw,strtrim((c.atlfile), 2 ),';','ATLFILE', format=f1
printf,nw,strtrim((c.natl), 2 ),';','NATL', format=f1
printf,nw,strtrim((c.atlscale), 2 ),';','ATLSCALE', format=f1
printf,nw,strtrim((c.atlflag), 2 ),';','ATLFLAG', format=f1
printf,nw,strtrim((c.atlaline), 2 ),';','ATLALINE', format=f1


printf,nw,strtrim((c.narcline), 2 ),';','NARCLINE', format=f1
for i = 0,c.narcline-1 do begin
   printf,nw,strtrim((c.acalwl(i)), 2 ),';','ACALWL' ,format=f1
   printf,nw,strtrim((c.apixc(i)), 2 ) ,';','APIXC'  ,format=f1
   printf,nw,strtrim((c.apixw(i)), 2 ) ,';','APIXW'  ,format=f1
   printf,nw,strtrim((c.aflag(i)), 2 ) ,';','AFLAG'  ,format=f1
endfor
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
close,nw   &    free_lun,nw
end
